"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.valueProjection = exports.subexpression = exports.slice = exports.projection = exports.pipe = exports.orExpression = exports.notExpression = exports.multiSelectList = exports.multiSelectObject = exports.literal = exports.keyValPair = exports.indexExpression = exports.index = exports.identity = exports.functionExpression = exports.flatten = exports.filterProjection = exports.field = exports.expref = exports.currentNode = exports.comparator = exports.andExpression = void 0;
/**
 * AST node representing a comparator expression.
 *
 * A comparator expression is a binary expression that compares two values.
 *
 * @param name The name of the comparator
 * @param first The left-hand side of the comparator
 * @param second The right-hand side of the comparator
 */
const comparator = (name, first, second) => ({
    type: 'comparator',
    children: [first, second],
    value: name,
});
exports.comparator = comparator;
/**
 * AST node representing the current node.
 *
 * The current node is a reference to the current value being processed.
 * In JMESPath, the current node is represented by the `@` symbol.
 */
const currentNode = () => ({
    type: 'current',
    children: [],
});
exports.currentNode = currentNode;
/**
 * AST node representing an expression reference.
 *
 * An expression reference is a reference to another expression.
 * In JMESPath, an expression reference is represented by the `&` symbol.
 *
 * @param expression The expression to reference
 */
const expref = (expression) => ({
    type: 'expref',
    children: [expression],
});
exports.expref = expref;
/**
 * AST node representing a function expression.
 *
 * A function expression is a reference to a function and its arguments.
 * The JMESPath specification defines a set of built-in functions that can
 * be used in expressions like `length(@)`, `map(@, &foo)`, etc.
 *
 * Custom functions can be added by extending the `Functions` class.
 *
 * @param name The name of the function
 * @param args The arguments to the function
 */
const functionExpression = (name, args) => ({
    type: 'function_expression',
    children: args,
    value: name,
});
exports.functionExpression = functionExpression;
/**
 * AST node representing a field reference.
 *
 * A field reference is a reference to a field in an object.
 */
const field = (name) => ({
    type: 'field',
    children: [],
    value: name,
});
exports.field = field;
/**
 * AST node representing a filter projection.
 *
 * A filter projection is a binary expression that filters the left-hand side
 * based on the right-hand side.
 *
 * In JMESPath, a filter projection is represented by the `[]` operator.
 * For example, `people[?age > 18]` filters the `people` array based on the
 * `age` field.
 *
 * @param left The left-hand side of the filter projection
 * @param right The right-hand side of the filter projection
 * @param comparator The comparator to use for the filter
 */
const filterProjection = (left, right, comparator) => ({
    type: 'filter_projection',
    children: [left, right, comparator],
});
exports.filterProjection = filterProjection;
/**
 * AST node representing a flatten expression.
 *
 * A flatten expression is a unary expression that flattens an array of arrays
 * into a single array.
 *
 * In JMESPath, a flatten expression is represented by the `[]` operator.
 * For example, `people[].name` flattens the `people` array and returns the
 * `name` field of each object in the array.
 *
 * @param node The node to flatten
 */
const flatten = (node) => ({
    type: 'flatten',
    children: [node],
});
exports.flatten = flatten;
/**
 * AST node representing an identity expression.
 */
const identity = () => ({ type: 'identity', children: [] });
exports.identity = identity;
/**
 * AST node representing an index reference.
 *
 * An index reference is a reference to an index in an array.
 * For example, `people[0]` references the first element in the `people` array.
 *
 * @param index The index to reference
 */
const index = (index) => ({
    type: 'index',
    value: index,
    children: [],
});
exports.index = index;
/**
 * AST node representing an index expression.
 *
 * An index expression holds the index and the children of the expression.
 *
 * @param children The children of the index expression
 */
const indexExpression = (children) => ({
    type: 'index_expression',
    children: children,
});
exports.indexExpression = indexExpression;
/**
 * AST node representing a key-value pair.
 *
 * @param keyName The name of the key
 * @param node The value of the key
 */
const keyValPair = (keyName, node) => ({
    type: 'key_val_pair',
    children: [node],
    value: keyName,
});
exports.keyValPair = keyValPair;
/**
 * AST node representing a literal value.
 *
 * A literal value is a value that is not a reference to another node.
 *
 * @param literalValue The value of the literal
 */
const literal = (literalValue) => ({
    type: 'literal',
    value: literalValue,
    children: [],
});
exports.literal = literal;
/**
 * AST node representing a multi-select object.
 *
 * A multi-select object is a reference to multiple nodes in an object.
 *
 * @param nodes
 */
const multiSelectObject = (nodes) => ({
    type: 'multi_select_object',
    children: nodes,
});
exports.multiSelectObject = multiSelectObject;
/**
 * AST node representing a multi-select list.
 *
 * @param nodes
 */
const multiSelectList = (nodes) => ({
    type: 'multi_select_list',
    children: nodes,
});
exports.multiSelectList = multiSelectList;
/**
 * AST node representing an or expression.
 *
 * @param left The left-hand side of the or expression
 * @param right The right-hand side of the or expression
 */
const orExpression = (left, right) => ({
    type: 'or_expression',
    children: [left, right],
});
exports.orExpression = orExpression;
/**
 * AST node representing an and expression.
 *
 * @param left The left-hand side of the and expression
 * @param right The right-hand side of the and expression
 */
const andExpression = (left, right) => ({
    type: 'and_expression',
    children: [left, right],
});
exports.andExpression = andExpression;
/**
 * AST node representing a not expression.
 *
 * @param left The left-hand side of the not expression
 * @param right The right-hand side of the not expression
 */
const notExpression = (expr) => ({
    type: 'not_expression',
    children: [expr],
});
exports.notExpression = notExpression;
/**
 * AST node representing a pipe expression.
 *
 * @param left The left-hand side of the pipe expression
 * @param right The right-hand side of the pipe expression
 */
const pipe = (left, right) => ({
    type: 'pipe',
    children: [left, right],
});
exports.pipe = pipe;
/**
 * AST node representing a projection.
 *
 * @param left The left-hand side of the projection
 * @param right The right-hand side of the projection
 */
const projection = (left, right) => ({
    type: 'projection',
    children: [left, right],
});
exports.projection = projection;
/**
 * AST node representing a subexpression.
 *
 * @param children The children of the subexpression
 */
const subexpression = (children) => ({
    type: 'subexpression',
    children: children,
});
exports.subexpression = subexpression;
/**
 * AST node representing a slice.
 *
 * A slice is a reference to a range of values in an array.
 *
 * @param start The start of the slice
 * @param end The end of the slice
 * @param step The step of the slice
 */
const slice = (start, end, step) => ({
    type: 'slice',
    children: [start, end, step],
});
exports.slice = slice;
/**
 * AST node representing a value projection.
 *
 * @param left The left-hand side of the value projection
 * @param right The right-hand side of the value projection
 */
const valueProjection = (left, right) => ({
    type: 'value_projection',
    children: [left, right],
});
exports.valueProjection = valueProjection;
